#include "MP3_Player_objects.h"
#include "MP3_Player_resources.h"
#include "built_in.h"


// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATA9_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISA9_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1000;
char PenDown;
typedef unsigned long TPointer;
TPointer PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TCLabel *local_clabel;
TCLabel *exec_clabel;
int clabel_order;
TImage *local_image;
TImage *exec_image;
int image_order;
TBox *local_box;
TBox *exec_box;
int box_order;
TBox_Round *local_round_box;
TBox_Round *exec_round_box;
int box_round_order;
TProgressBar *local_progress_bar;
TProgressBar *exec_progress_bar;
int progress_bar_order;

void PMPWaitBusy() {
  while(PMMODEbits.BUSY);
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  PMDIN = index;
  PMPWaitBusy();
}

void Write_Command( unsigned short cmd ) {
  TFT_RS = 1;
  PMDIN = cmd;
  PMPWaitBusy();
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  PMDIN = _data;
  PMPWaitBusy();
}


void Init_ADC() {
  AD1PCFG = 0xFFFF;
  PCFG12_bit = 0;
  PCFG13_bit = 0;
  // PMP setup
  ADC1_Init();
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TImage               PauseButton;
  TImage               PlayButton;
  TBox                   ProgressMasque;
  TBox                   MasqueVolume;
  TProgressBar                 ProgressBar1;
char ProgressBar1_Caption[6] = "";

  TLine                  VolumeLine;
  TBox_Round             VolumeButton;
  TLabel                 TextVolume;
char TextVolume_Caption[5] = "30%";

  TBox_Round             ProgressButton;
  TImage               StopButton;
  TImage               ForwardButton;
  TImage               BackButton;
  TLine                  Line1;
  TCLabel          Label1 = 
         {
         &Screen1              , //   Label1.OwnerScreen
         13                    , //   Label1.Order          
         132                   , //   Label1.Left           
         4                     , //   Label1.Top             
         55                    , //   Label1.Width         
         15                    , //   Label1.Height       
         &Label1_Caption       , //   Label1.Caption        
         &Tahoma11x13_Regular  , //   Label1.FontName       
         0x0000                , //   Label1.Font_Color     
         1                     , //   Label1.Visible        
         1                     , //   Label1.Active        
         0                     , //   Label1.OnUpPtr
         0                     , //   Label1.OnDownPtr
         0                     , //   Label1.OnClickPtr
         0                       //   Label1.OnPressPtr
         };
const char Label1_Caption[11] = "MP3 Player";

  TLabel                 * const code Screen1_Labels[1]=
         {
         &TextVolume           
         };
  TCLabel                * const code Screen1_CLabels[1]=
         {
         &Label1               
         };
  TImage                 * const code Screen1_Images[5]=
         {
         &PauseButton,         
         &PlayButton,          
         &StopButton,          
         &ForwardButton,       
         &BackButton           
         };
  TBox                   * const code Screen1_Boxes[2]=
         {
         &ProgressMasque,      
         &MasqueVolume         
         };
  TBox_Round             * const code Screen1_Boxes_Round[2]=
         {
         &VolumeButton,        
         &ProgressButton       
         };
  TLine                  * const code Screen1_Lines[2]=
         {
         &VolumeLine,          
         &Line1                
         };
  TProgressBar           * const code Screen1_ProgressBars[1]=
         {
         &ProgressBar1         
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.LabelsCount               = 1;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.CLabelsCount              = 1;
  Screen1.CLabels                   = Screen1_CLabels;
  Screen1.ImagesCount               = 5;
  Screen1.Images                    = Screen1_Images;
  Screen1.BoxesCount                = 2;
  Screen1.Boxes                     = Screen1_Boxes;
  Screen1.Boxes_RoundCount          = 2;
  Screen1.Boxes_Round               = Screen1_Boxes_Round;
  Screen1.LinesCount                = 2;
  Screen1.Lines                     = Screen1_Lines;
  Screen1.ProgressBarsCount         = 1;
  Screen1.ProgressBars              = Screen1_ProgressBars;
  Screen1.ObjectsCount              = 14;


  PauseButton.OwnerScreen     = &Screen1;
  PauseButton.Order           = 0;
  PauseButton.Left            = 151;
  PauseButton.Top             = 148;
  PauseButton.Width           = 60;
  PauseButton.Height          = 60;
  PauseButton.Picture_Type    = 1;
  PauseButton.Picture_Ratio   = 1;
  PauseButton.Picture_Name    = pause_jpg;
  PauseButton.Visible         = 0;
  PauseButton.Active          = 0;
  PauseButton.OnUpPtr         = 0;
  PauseButton.OnDownPtr       = 0;
  PauseButton.OnClickPtr      = ButtonRound1OnClick;
  PauseButton.OnPressPtr      = 0;

  PlayButton.OwnerScreen     = &Screen1;
  PlayButton.Order           = 1;
  PlayButton.Left            = 150;
  PlayButton.Top             = 148;
  PlayButton.Width           = 60;
  PlayButton.Height          = 60;
  PlayButton.Picture_Type    = 1;
  PlayButton.Picture_Ratio   = 1;
  PlayButton.Picture_Name    = play_jpg;
  PlayButton.Visible         = 1;
  PlayButton.Active          = 1;
  PlayButton.OnUpPtr         = 0;
  PlayButton.OnDownPtr       = 0;
  PlayButton.OnClickPtr      = ButtonRound1OnClick;
  PlayButton.OnPressPtr      = 0;

  ProgressMasque.OwnerScreen     = &Screen1;
  ProgressMasque.Order           = 2;
  ProgressMasque.Left            = 0;
  ProgressMasque.Top             = 218;
  ProgressMasque.Width           = 320;
  ProgressMasque.Height          = 20;
  ProgressMasque.Pen_Width       = 0;
  ProgressMasque.Pen_Color       = 0x0000;
  ProgressMasque.Visible         = 1;
  ProgressMasque.Active          = 1;
  ProgressMasque.Transparent     = 1;
  ProgressMasque.Gradient        = 0;
  ProgressMasque.Gradient_Orientation = 0;
  ProgressMasque.Gradient_Start_Color = 0xFFFF;
  ProgressMasque.Gradient_End_Color = 0xC618;
  ProgressMasque.Color           = 0xFFFF;
  ProgressMasque.PressColEnabled = 1;
  ProgressMasque.Press_Color     = 0xE71C;
  ProgressMasque.OnUpPtr         = 0;
  ProgressMasque.OnDownPtr       = 0;
  ProgressMasque.OnClickPtr      = 0;
  ProgressMasque.OnPressPtr      = ProgressMasqueOnPress;

  MasqueVolume.OwnerScreen     = &Screen1;
  MasqueVolume.Order           = 3;
  MasqueVolume.Left            = 291;
  MasqueVolume.Top             = 45;
  MasqueVolume.Width           = 20;
  MasqueVolume.Height          = 110;
  MasqueVolume.Pen_Width       = 0;
  MasqueVolume.Pen_Color       = 0x0000;
  MasqueVolume.Visible         = 1;
  MasqueVolume.Active          = 1;
  MasqueVolume.Transparent     = 1;
  MasqueVolume.Gradient        = 0;
  MasqueVolume.Gradient_Orientation = 0;
  MasqueVolume.Gradient_Start_Color = 0xFFFF;
  MasqueVolume.Gradient_End_Color = 0xC618;
  MasqueVolume.Color           = 0xFFFF;
  MasqueVolume.PressColEnabled = 0;
  MasqueVolume.Press_Color     = 0xE71C;
  MasqueVolume.OnUpPtr         = 0;
  MasqueVolume.OnDownPtr       = 0;
  MasqueVolume.OnClickPtr      = 0;
  MasqueVolume.OnPressPtr      = MasqueVolumeOnPress;

  ProgressBar1.OwnerScreen     = &Screen1;
  ProgressBar1.Order           = 4;
  ProgressBar1.Left            = 10;
  ProgressBar1.Top             = 225;
  ProgressBar1.Width           = 300;
  ProgressBar1.Height          = 6;
  ProgressBar1.Pen_Width       = 1;
  ProgressBar1.Pen_Color       = 0x0000;
  ProgressBar1.Visible         = 1;
  ProgressBar1.Transparent     = 1;
  ProgressBar1.Caption         = ProgressBar1_Caption;
  ProgressBar1.FontName        = Tahoma11x13_Regular;
  ProgressBar1.Font_Color      = 0x0000;
  ProgressBar1.Gradient        = 1;
  ProgressBar1.Gradient_Orientation = 0;
  ProgressBar1.Gradient_Start_Color = 0xFFFF;
  ProgressBar1.Gradient_End_Color = 0x07E0;
  ProgressBar1.Color           = 0xA65E;
  ProgressBar1.Background_Color = 0x8410;
  ProgressBar1.Min             = 0;
  ProgressBar1.Max             = 100;
  ProgressBar1.Position        = 0;
  ProgressBar1.Show_Position   = 0;
  ProgressBar1.Show_Percentage = 0;
  ProgressBar1.Smooth          = 1;
  ProgressBar1.Rounded         = 1;
  ProgressBar1.Corner_Radius   = 1;

  VolumeLine.OwnerScreen     = &Screen1;
  VolumeLine.Order           = 5;
  VolumeLine.First_Point_X   = 301;
  VolumeLine.First_Point_Y   = 50;
  VolumeLine.Second_Point_X  = 301;
  VolumeLine.Second_Point_Y  = 150;
  VolumeLine.Visible         = 1;
  VolumeLine.Pen_Width       = 1;
  VolumeLine.Color           = 0x0000;

  VolumeButton.OwnerScreen     = &Screen1;
  VolumeButton.Order           = 6;
  VolumeButton.Left            = 292;
  VolumeButton.Top             = 116;
  VolumeButton.Width           = 18;
  VolumeButton.Height          = 10;
  VolumeButton.Pen_Width       = 1;
  VolumeButton.Pen_Color       = 0x0000;
  VolumeButton.Visible         = 1;
  VolumeButton.Active          = 1;
  VolumeButton.Transparent     = 1;
  VolumeButton.Gradient        = 1;
  VolumeButton.Gradient_Orientation = 0;
  VolumeButton.Gradient_Start_Color = 0xFFFF;
  VolumeButton.Gradient_End_Color = 0xC618;
  VolumeButton.Color           = 0xC618;
  VolumeButton.PressColEnabled = 1;
  VolumeButton.Press_Color     = 0xE71C;
  VolumeButton.Corner_Radius   = 3;
  VolumeButton.OnUpPtr         = VolumeButtonOnUp;
  VolumeButton.OnDownPtr       = 0;
  VolumeButton.OnClickPtr      = 0;
  VolumeButton.OnPressPtr      = MasqueVolumeOnPress;

  TextVolume.OwnerScreen     = &Screen1;
  TextVolume.Order           = 7;
  TextVolume.Left            = 285;
  TextVolume.Top             = 160;
  TextVolume.Width           = 23;
  TextVolume.Height          = 15;
  TextVolume.Visible         = 1;
  TextVolume.Active          = 1;
  TextVolume.Caption         = TextVolume_Caption;
  TextVolume.FontName        = Tahoma11x13_Regular;
  TextVolume.Font_Color      = 0x0000;
  TextVolume.OnUpPtr         = 0;
  TextVolume.OnDownPtr       = 0;
  TextVolume.OnClickPtr      = 0;
  TextVolume.OnPressPtr      = 0;

  ProgressButton.OwnerScreen     = &Screen1;
  ProgressButton.Order           = 8;
  ProgressButton.Left            = 5;
  ProgressButton.Top             = 219;
  ProgressButton.Width           = 10;
  ProgressButton.Height          = 18;
  ProgressButton.Pen_Width       = 1;
  ProgressButton.Pen_Color       = 0x0000;
  ProgressButton.Visible         = 1;
  ProgressButton.Active          = 1;
  ProgressButton.Transparent     = 1;
  ProgressButton.Gradient        = 1;
  ProgressButton.Gradient_Orientation = 0;
  ProgressButton.Gradient_Start_Color = 0xFFFF;
  ProgressButton.Gradient_End_Color = 0xC618;
  ProgressButton.Color           = 0xC618;
  ProgressButton.PressColEnabled = 1;
  ProgressButton.Press_Color     = 0xE71C;
  ProgressButton.Corner_Radius   = 3;
  ProgressButton.OnUpPtr         = 0;
  ProgressButton.OnDownPtr       = 0;
  ProgressButton.OnClickPtr      = 0;
  ProgressButton.OnPressPtr      = 0;

  StopButton.OwnerScreen     = &Screen1;
  StopButton.Order           = 9;
  StopButton.Left            = 80;
  StopButton.Top             = 148;
  StopButton.Width           = 60;
  StopButton.Height          = 60;
  StopButton.Picture_Type    = 1;
  StopButton.Picture_Ratio   = 1;
  StopButton.Picture_Name    = stop_jpg;
  StopButton.Visible         = 1;
  StopButton.Active          = 1;
  StopButton.OnUpPtr         = 0;
  StopButton.OnDownPtr       = 0;
  StopButton.OnClickPtr      = StopButtonOnClick;
  StopButton.OnPressPtr      = 0;

  ForwardButton.OwnerScreen     = &Screen1;
  ForwardButton.Order           = 10;
  ForwardButton.Left            = 220;
  ForwardButton.Top             = 148;
  ForwardButton.Width           = 60;
  ForwardButton.Height          = 60;
  ForwardButton.Picture_Type    = 1;
  ForwardButton.Picture_Ratio   = 1;
  ForwardButton.Picture_Name    = forward_jpg;
  ForwardButton.Visible         = 1;
  ForwardButton.Active          = 1;
  ForwardButton.OnUpPtr         = 0;
  ForwardButton.OnDownPtr       = 0;
  ForwardButton.OnClickPtr      = 0;
  ForwardButton.OnPressPtr      = 0;

  BackButton.OwnerScreen     = &Screen1;
  BackButton.Order           = 11;
  BackButton.Left            = 10;
  BackButton.Top             = 148;
  BackButton.Width           = 60;
  BackButton.Height          = 60;
  BackButton.Picture_Type    = 1;
  BackButton.Picture_Ratio   = 1;
  BackButton.Picture_Name    = back_jpg;
  BackButton.Visible         = 1;
  BackButton.Active          = 1;
  BackButton.OnUpPtr         = 0;
  BackButton.OnDownPtr       = 0;
  BackButton.OnClickPtr      = 0;
  BackButton.OnPressPtr      = 0;

  Line1.OwnerScreen     = &Screen1;
  Line1.Order           = 12;
  Line1.First_Point_X   = 0;
  Line1.First_Point_Y   = 20;
  Line1.Second_Point_X  = 320;
  Line1.Second_Point_Y  = 20;
  Line1.Visible         = 1;
  Line1.Pen_Width       = 1;
  Line1.Color           = 0x0000;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetCLabel(index)              CurrentScreen->CLabels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetBox_Round(index)           CurrentScreen->Boxes_Round[index]
#define GetLine(index)                CurrentScreen->Lines[index]
#define GetProgressBar(index)         CurrentScreen->ProgressBars[index]

void DeleteTrailingSpaces(char* str){
  char counter;
  while(1) {
  if (str[0] == ' ') {
    for(counter = 0; counter < strlen(str); counter++) {
      str[counter] = str[counter+1];
    }
  }
  else
    break;
  }
}


void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawCLabel(TCLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawImage(TImage *AImage) {
  if (AImage->Visible) {
    TFT_Image_Jpeg(AImage->Left, AImage->Top, AImage->Picture_Name);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawRoundBox(TBox_Round *Around_box) {
    if (Around_box->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_box->Transparent, Around_box->Press_Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_End_Color, Around_box->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_box->Transparent, Around_box->Color, Around_box->Gradient, Around_box->Gradient_Orientation,
                      Around_box->Gradient_Start_Color, Around_box->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_box->Pen_Color, Around_box->Pen_Width);
      TFT_Rectangle_Round_Edges(Around_box->Left + 1, Around_box->Top + 1,
        Around_box->Left + Around_box->Width - 2,
        Around_box->Top + Around_box->Height - 2, Around_box->Corner_Radius);
    }
}

void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

void UpdatePBPosition(TProgressBar *AProgressBar) {
  int move_offset;
  unsigned int locPosition = 0;
  unsigned cnt = 0;
  char tmpStr[8];
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  move_offset = 10;
  locPosition = (AProgressBar->Position - AProgressBar->Min) * 100 / (AProgressBar->Max - AProgressBar->Min);
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Background_Color, AProgressBar->Pen_Width);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    if (caption_height >  AProgressBar->Height)
        caption_height =  AProgressBar->Height;
    if (caption_length >  AProgressBar->Width)
        caption_length =  AProgressBar->Width;
    if (AProgressBar->Position < (AProgressBar->Left + (AProgressBar->Width - caption_length) / 2)) {
    TFT_Rectangle(AProgressBar->Left + (AProgressBar->Width - caption_length) / 2,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2,
                  AProgressBar->Left + (AProgressBar->Width - caption_length) / 2 + caption_length,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2 + caption_height);
    }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius  + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100 , AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      if (locPosition > 0) {
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
        while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
              if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
              else
          TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
          cnt = cnt + move_offset + 3;
        }
      }
    // Clear caption frist
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
    }
    else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
    }
    TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
    }
 }
}

void DrawProgressBar(TProgressBar *AProgressBar) {
  unsigned long locPosition = 0;
  int move_offset = 0;
  unsigned cnt = 0;
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  unsigned int _tmpI, _tmpII;
  char tmpStr[8];
  move_offset = 10;
  _tmpI = AProgressBar->Position - AProgressBar->Min;
  _tmpII = AProgressBar->Max - AProgressBar->Min;
  locPosition = (unsigned long)_tmpI * 100 / _tmpII;
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      // First draw back rect
      TFT_Rectangle_Round_Edges(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height, AProgressBar->Corner_Radius);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius   + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      // First draw back rect
      TFT_Rectangle(AProgressBar->Left, AProgressBar->Top, AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
     while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
            if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
            else
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
        cnt = cnt + move_offset + 3;
     }
    }
    // Clear caption first
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
       }
       else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
       }
       TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
       TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
       TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
     }
 }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short clabel_idx;
  TCLabel *local_clabel;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short round_box_idx;
  TBox_Round *local_round_box;
  unsigned short line_idx;
  TLine *local_line;
  unsigned short progress_bar_idx;
  TProgressBar *local_progress_bar;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  label_idx = 0;
  clabel_idx = 0;
  image_idx = 0;
  box_idx = 0;
  round_box_idx = 0;
  line_idx = 0;
  progress_bar_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (clabel_idx < CurrentScreen->CLabelsCount) {
      local_clabel = GetCLabel(clabel_idx);
      if (order == local_clabel->Order) {
        clabel_idx++;
        order++;
        DrawCLabel(local_clabel);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (round_box_idx < CurrentScreen->Boxes_RoundCount) {
      local_round_box = GetBox_Round(round_box_idx);
      if (order == local_round_box->Order) {
        round_box_idx++;
        order++;
        DrawRoundBox(local_round_box);
      }
    }

    if (line_idx < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

    if (image_idx < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

    if (progress_bar_idx < CurrentScreen->ProgressBarsCount) {
      local_progress_bar = GetProgressBar(progress_bar_idx);
      if (order == local_progress_bar->Order) {
        progress_bar_idx++;
        order++;
        DrawProgressBar(local_progress_bar);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  label_order         = -1;
  clabel_order        = -1;
  image_order         = -1;
  box_order           = -1;
  box_round_order     = -1;
  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  CLabels
  for ( _object_count = 0 ; _object_count < CurrentScreen->CLabelsCount ; _object_count++ ) {
    local_clabel = GetCLabel(_object_count);
    if (local_clabel->Active == 1) {
      if (IsInsideObject(X, Y, local_clabel->Left, local_clabel->Top,
                         local_clabel->Width, local_clabel->Height) == 1) {
        clabel_order = local_clabel->Order;
        exec_clabel = local_clabel;
      }
    }
  }

  //  Images
  for ( _object_count = 0 ; _object_count < CurrentScreen->ImagesCount ; _object_count++ ) {
    local_image = GetImage(_object_count);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  //  Boxes with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->Boxes_RoundCount ; _object_count++ ) {
    local_round_box = GetBox_Round(_object_count);
    if (local_round_box->Active == 1) {
      if (IsInsideObject(X, Y, local_round_box->Left, local_round_box->Top,
                         local_round_box->Width, local_round_box->Height) == 1) {
        box_round_order = local_round_box->Order;
        exec_round_box = local_round_box;
      }
    }
  }

  _object_count = -1;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (clabel_order >  _object_count )
    _object_count = clabel_order;
  if (image_order >  _object_count )
    _object_count = image_order;
  if (box_order >  _object_count )
    _object_count = box_order;
  if (box_round_order >  _object_count )
    _object_count = box_round_order;
}


static void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_label          = 0;
  exec_clabel         = 0;
  exec_image          = 0;
  exec_box            = 0;
  exec_round_box      = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnPressPtr != 0) {
          exec_clabel->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->OnPressPtr != 0) {
          exec_round_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled == 1) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
    // Round Box
    case 7: {
      if (PressedObject != 0) {
        exec_round_box = (TBox_Round*)PressedObject;
        if ((exec_round_box->PressColEnabled == 1) && (exec_round_box->OwnerScreen == CurrentScreen)) {
          DrawRoundBox(exec_round_box);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;
  exec_clabel          = 0;
  exec_image          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (TPointer)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // CLabels
    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnUpPtr != 0)
          exec_clabel->OnUpPtr();
        if (PressedObject == (TPointer)exec_clabel)
          if (exec_clabel->OnClickPtr != 0)
            exec_clabel->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (TPointer)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (TPointer)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes with Round Edges
    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->OnUpPtr != 0)
          exec_round_box->OnUpPtr();
        if (PressedObject == (TPointer)exec_round_box)
          if (exec_round_box->OnClickPtr != 0)
            exec_round_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_label          = 0;
  exec_clabel         = 0;
  exec_image          = 0;
  exec_box            = 0;
  exec_round_box      = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (TPointer)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        PressedObject = (TPointer)exec_clabel;
        PressedObjectType = 10;
        if (exec_clabel->OnDownPtr != 0) {
          exec_clabel->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (TPointer)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (TPointer)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_round_order) {
      if (exec_round_box->Active == 1) {
        if (exec_round_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawRoundBox(exec_round_box);
        }
        PressedObject = (TPointer)exec_round_box;
        PressedObjectType = 7;
        if (exec_round_box->OnDownPtr != 0) {
          exec_round_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  CM1CON  = 0;                    // Turn off comparators
  CM2CON  = 0;
  AD1PCFG = 0xff;                 // Make all inputs digital

  PMMODE = 0;
  PMAEN  = 0;
  PMCON  = 0;  // WRSP: Write Strobe Polarity bit
  PMMODEbits.MODE = 2;     // Master 2
  PMMODEbits.WAITB = 0;
  PMMODEbits.WAITM = 1;
  PMMODEbits.WAITE = 0;
  PMMODEbits.MODE16 = 1;   // 16 bit mode
  PMCONbits.CSF = 0;
  PMCONbits.PTRDEN = 1;
  PMCONbits.PTWREN = 1;
  PMCONbits.PMPEN = 1;
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(76, 907, 77, 915);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
