typedef struct Screen TScreen;

typedef struct  Label {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          *Caption;
  const char    *FontName;
  unsigned int  Font_Color;
  char          Visible;
  char          Active;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TLabel;

typedef const struct  CLabel {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  const char          *Caption;
  const char    *FontName;
  unsigned int  Font_Color;
  char          Visible;
  char          Active;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TCLabel;

typedef struct  Image {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  const char    *Picture_Name;
  char          Visible;
  char          Active;
  char          Picture_Type;
  char          Picture_Ratio;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TImage;

typedef struct  Box {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TBox;

typedef struct  Box_Round {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          Corner_Radius;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TBox_Round;

typedef struct  Line {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  First_Point_X;
  unsigned int  First_Point_Y;
  unsigned int  Second_Point_X;
  unsigned int  Second_Point_Y;
  char          Pen_Width;
  char          Visible;
  unsigned int  Color;
} TLine;

typedef struct ProgressBar {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Transparent;
  char          *Caption;
  const char    *FontName;
  unsigned int  Font_Color;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  unsigned int  Background_Color;
  unsigned int  Min;
  unsigned int  Max;
  unsigned int  Position;
  char          Show_Position;
  char          Show_Percentage;
  char          Smooth;
  char          Rounded;
  char          Corner_Radius;
} TProgressBar;

struct Screen {
  unsigned int           Color;
  unsigned int           Width;
  unsigned int           Height;
  unsigned int         ObjectsCount;
  unsigned int           LabelsCount;
  TLabel                 * const code *Labels;
  unsigned int           CLabelsCount;
  TCLabel                * const code *CLabels;
  unsigned int           ImagesCount;
  TImage                 * const code *Images;
  unsigned int           BoxesCount;
  TBox                   * const code *Boxes;
  unsigned int           Boxes_RoundCount;
  TBox_Round             * const code *Boxes_Round;
  unsigned int           LinesCount;
  TLine                  * const code *Lines;
  unsigned int           ProgressBarsCount;
  TProgressBar           * const code *ProgressBars;
};

extern   TScreen                Screen1;
extern   TImage               PauseButton;
extern   TImage               PlayButton;
extern   TBox                   ProgressMasque;
extern   TBox                   MasqueVolume;
extern   TProgressBar                 ProgressBar1;
extern   TLine                  VolumeLine;
extern   TBox_Round             VolumeButton;
extern   TLabel                 TextVolume;
extern   TBox_Round             ProgressButton;
extern   TImage               StopButton;
extern   TImage               ForwardButton;
extern   TImage               BackButton;
extern   TLine                  Line1;
extern   TCLabel          Label1;
extern   TLabel                 * const code Screen1_Labels[1];
extern   TCLabel                * const code Screen1_CLabels[1];
extern   TImage                 * const code Screen1_Images[5];
extern   TBox                   * const code Screen1_Boxes[2];
extern   TBox_Round             * const code Screen1_Boxes_Round[2];
extern   TLine                  * const code Screen1_Lines[2];
extern   TProgressBar           * const code Screen1_ProgressBars[1];


/////////////////////////
// Events Code Declarations
void ButtonRound1OnClick();
void MasqueVolumeOnPress();
void ProgressMasqueOnPress();
void StopButtonOnClick();
void VolumeButtonOnUp();
/////////////////////////

/////////////////////////////////
// Caption variables Declarations
extern char PauseButton_Caption[];
extern char PlayButton_Caption[];
extern char ProgressMasque_Caption[];
extern char MasqueVolume_Caption[];
extern char ProgressBar1_Caption[];
extern char VolumeLine_Caption[];
extern char VolumeButton_Caption[];
extern char TextVolume_Caption[];
extern char ProgressButton_Caption[];
extern char StopButton_Caption[];
extern char ForwardButton_Caption[];
extern char BackButton_Caption[];
extern char Line1_Caption[];
extern const char Label1_Caption[];
/////////////////////////////////

void DrawScreen(TScreen *aScreen);
void DrawLabel(TLabel *ALabel);
void DrawCLabel(TCLabel *AClabel);
void DrawImage(TImage *AImage);
void DrawBox(TBox *ABox);
void DrawRoundBox(TBox_Round *Around_box);
void DrawLine(TLine *Aline);
void DrawProgressBar(TProgressBar *AProgressBar);
void UpdatePBPosition(TProgressBar *AProgressBar);
void Check_TP();
void Start_TP();
