/**************************************************************************************************
* File: MP3_driver.h
* File Type: C - Header File
* Project Name: BrmBrrrrm
* Company: (c) mikroElektronika, 2011
* Revision History:
*     20101121 (TL):
*       - initial release;
*     20110211 (FJ):
*       - fixed write/read to the shared SFR;
*     20110512:
*       - modified for HW revision 1.05 (FJ);
*     20111020:
*       - modified for mikroMedia PIC32 HW revision 1.10 (DA);
* Description:
*     This module contains a set of functions that are used for communication with
*     VS1053E mp3 codec.
* Target:
*       MCU:             P32MX460F512L
*       Dev.Board:       MikroMMB_for_PIC32
*                        http://www.mikroe.com/eng/products/view/595/mikrommb-for-pic32-board/
*       Oscillator:      80000000 Hz (80MHz)
*       SW:              mikroC PRO for PIC32
*                        http://www.pic32compilers.com/
* NOTES:
*     Mp3 codec use SPI bus to communicate with MCU.
**************************************************************************************************/

extern unsigned long file_size;
extern char * mp3_buffer;
extern char volume_left, volume_right;

extern const char WRITE_CODE           ;
extern const char READ_CODE            ;

extern const char SCI_BASE_ADDR        ;
extern const char SCI_MODE_ADDR        ;
extern const char SCI_STATUS_ADDR      ;
extern const char SCI_BASS_ADDR        ;
extern const char SCI_CLOCKF_ADDR      ;
extern const char SCI_DECODE_TIME_ADDR ;
extern const char SCI_AUDATA_ADDR      ;
extern const char SCI_WRAM_ADDR        ;
extern const char SCI_WRAMADDR_ADDR    ;
extern const char SCI_HDAT0_ADDR       ;
extern const char SCI_HDAT1_ADDR       ;
extern const char SCI_AIADDR_ADDR      ;
extern const char SCI_VOL_ADDR         ;
extern const char SCI_AICTRL0_ADDR     ;
extern const char SCI_AICTRL1_ADDR     ;
extern const char SCI_AICTRL2_ADDR     ;
extern const char SCI_AICTRL3_ADDR     ;

/**************************************************************************************************
* MMC Chip Select connection
**************************************************************************************************/
extern sbit Mmc_Chip_Select;
extern sbit Mmc_Chip_Select_Direction;

/**************************************************************************************************
* Serial Flash Chip Select connection
**************************************************************************************************/
extern sbit CS_Serial_Flash_bit;
extern sbit CS_Serial_Flash_Direction_bit;

/**************************************************************************************************
* CODEC V1053E connections
**************************************************************************************************/
extern sbit MP3_CS_Direction;
extern sbit MP3_CS;
extern sbit MP3_RST_Direction;
extern sbit MP3_RST;
extern sbit DREQ_Direction;
extern sbit DREQ;
extern sbit BSYNC_Direction;
extern sbit BSYNC;

extern int playing;

void MP3_Start(void);
void MP3_Init(void);
int StartPlayingFile(char * fileName);

// Writes one byte to MP3 SCI
void MP3_SCI_Write(char address, unsigned int data_in);
// Reads words_count words from MP3 SCI
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer);
// Write one byte to MP3 SDI
void MP3_SDI_Write(char data_);
// Write 32 bytes to MP3 SDI
void MP3_SDI_Write_32(char *data_);
// Set volume
void MP3_Set_Volume(char left, char right);

/**************************************************************************************************
* End of File
**************************************************************************************************/